package net.minecraft.src;

import java.io.FileInputStream;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import net.minecraft.server.MinecraftServer;

// SDK FPS
@SuppressWarnings({ "rawtypes", "unchecked" })
public class mod_SdkFps extends SdkBaseModMp
{
	// Blocks
    public static Block blockPlayerSpawner;
    public static Block blockWeaponSpawnerGuns;
    public static Block blockWeaponSpawnerNotGuns;
    
    // Items
    public static Item itemPlayerSpawner;
    public static Item itemWeaponSpawnerGuns;
    public static Item itemWeaponSpawnerNotGuns;
    
    // Textures
    public static final int blockSpawnerSideIndexInTexture = ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerSide.png");
    public static int[] weaponSpawnerGunsTextureIndices = {
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSentryAk47.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSentryMp5.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSentryShotgun.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSentryDeagle.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSentryRocketLauncher.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSentryRocketLauncherLaser.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSentrySniper.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSentryFlamethrower.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSentrySg552.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSentryMinigun.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSentryLaser.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSentryM4.png")
    };
    public static int[] weaponSpawnerNotGunsTextureIndices = {
    	ModLoader.addOverride("/terrain.png", "/sdk/blockWeaponSpawnerNotGuns.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerGrenade.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerGrenadeStun.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerGrenadeSmoke.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerParachute.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerGrapplingHook.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerGrenadeSticky.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerNightvision.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerScubaTank.png"),
    	ModLoader.addOverride("/terrain.png", "/sdk/blockSpawnerGrenadeIncendiary.png"),
    };
    
    // Properties
    public static boolean blocksMineable = true;
    public static boolean blocksExplodable = true;
    public static boolean blocksBurnable = true;
    public static boolean enablePlayerSpawners = true;
    public static boolean useRandomPlayerSpawner = true;
    public static boolean enableWeaponSpawners = true;
    public static boolean enableSpawnerRecipes = false;
    public static boolean restrictInventory = false;
    public static boolean respawnWithPistol = false;
    public static int weaponSpawnerRespawnDelay = 30;
    public static boolean playersDropInventory = true;
    
    // For respawn
    private static ArrayList<SdkPoint3d<Integer>> spawnPoints = new ArrayList<SdkPoint3d<Integer>>();
    
    // Weapon maps
    public static Map<Byte, SdkItemGun> weaponGunsIdItemMap = new HashMap<Byte, SdkItemGun>();
    public static Map<Integer, Integer> weaponGunsIdTextureMap = new HashMap<Integer, Integer>();
    
    public static Map<Byte, Item> weaponNotGunsIdItemMap = new HashMap<Byte, Item>();
    public static Map<Integer, Integer> weaponNotGunsIdTextureMap = new HashMap<Integer, Integer>();
    
    private static Set<Integer> allowedItemIds = new HashSet<Integer>();
    private static Set<Integer> adminItemIds = new HashSet<Integer>();
    
    // For unbreakable blocks
	private static boolean areBlocksMineable = true;
	private static float[] blockHardness;
	private static boolean areBlocksExplodable = true;
	private static float[] blockResistance;
	private static boolean areBlocksBurnable = true;
	private static int[] blockChanceToEncourageFire;
	private static int[] blockAbilityToCatchFire;
    
    static
    {
    	loadProperties();
    }
    
    private static void loadProperties()
    {
    	Properties properties = new Properties();
    	try
    	{
    		String propertiesPath = mod_SdkFps.class.getProtectionDomain().getCodeSource().getLocation().toURI().getPath();
    		propertiesPath = propertiesPath.substring(0, propertiesPath.lastIndexOf('/'));
    		propertiesPath += "/mod_SdkFps.properties";
    		
    		try
        	{
	    		properties.load(new FileInputStream(propertiesPath));
	        }
	        catch (Exception ex)
	        {
				ModLoader.getLogger().throwing("mod_SdkFps", "loadProperties", ex);
				ModLoader.ThrowException("mod_SdkFps.properties not found in the server folder.", ex);
				return;
	        }
	        
    		String property;
    		
    		property = properties.getProperty("blocksMineable");
    		blocksMineable = Boolean.parseBoolean(property.trim());
    		
    		property = properties.getProperty("blocksExplodable");
    		blocksExplodable = Boolean.parseBoolean(property.trim());
    		
    		property = properties.getProperty("blocksBurnable");
    		blocksBurnable = Boolean.parseBoolean(property.trim());
    		
    		property = properties.getProperty("enablePlayerSpawners");
    		enablePlayerSpawners = Boolean.parseBoolean(property.trim());
    		
    		property = properties.getProperty("useRandomPlayerSpawner");
    		useRandomPlayerSpawner = Boolean.parseBoolean(property.trim());
    		
    		property = properties.getProperty("enableWeaponSpawners");
    		enableWeaponSpawners = Boolean.parseBoolean(property.trim());
    		
    		property = properties.getProperty("enableSpawnerRecipes");
    		enableSpawnerRecipes = Boolean.parseBoolean(property.trim());
    		
    		property = properties.getProperty("restrictInventory");
    		restrictInventory = Boolean.parseBoolean(property.trim());
    		
    		property = properties.getProperty("respawnWithPistol");
    		respawnWithPistol = Boolean.parseBoolean(property.trim());
    		
    		property = properties.getProperty("weaponSpawnerRespawnDelay");
    		weaponSpawnerRespawnDelay = Integer.parseInt(property.trim());
    		
    		property = properties.getProperty("playersDropInventory");
    		playersDropInventory = Boolean.parseBoolean(property.trim());
        }
    	catch (URISyntaxException ex)
    	{
    		ModLoader.getLogger().throwing("mod_SdkFps", "loadProperties", ex);
			SdkTools.ThrowException("An impossible error has occurred!", ex);
			return;
    	}
    	catch (Exception ex)
    	{
    		if(ex instanceof RuntimeException)
    			throw (RuntimeException)ex;
    		else
    		{
    			ModLoader.getLogger().throwing("mod_SdkFps", "loadProperties", ex);
    			SdkTools.ThrowException("mod_SdkFps.properties is corrupt.", ex);
    			return;
    		}
    	}
    }
    
    @Override
    public void InitBlocks()
    {
    	// Blocks
        blockPlayerSpawner = (new SdkBlockPlayerSpawner(117, ModLoader.addOverride("/terrain.png", "/sdk/blockPlayerSpawner.png"))).setHardness(6000000F).setResistance(6000000F).setStepSound(Block.soundMetalFootstep).setBlockName("blockPlayerSpawner");
        blockWeaponSpawnerGuns = (new SdkBlockWeaponSpawnerGuns(118, ModLoader.addOverride("/terrain.png", "/sdk/blockWeaponSpawnerGuns.png"))).setHardness(6000000F).setResistance(6000000F).setStepSound(Block.soundMetalFootstep).setBlockName("blockWeaponSpawnerGuns");
        blockWeaponSpawnerNotGuns = (new SdkBlockWeaponSpawnerNotGuns(119, weaponSpawnerNotGunsTextureIndices[0])).setHardness(6000000F).setResistance(6000000F).setStepSound(Block.soundMetalFootstep).setBlockName("blockWeaponSpawnerNotGuns");
		
		ModLoader.RegisterBlock(blockPlayerSpawner);
		ModLoader.RegisterBlock(blockWeaponSpawnerGuns);
		ModLoader.RegisterBlock(blockWeaponSpawnerNotGuns);
        
        // Items
        itemPlayerSpawner = new SdkItemUltimateTool(2528, blockPlayerSpawner).setIconIndex(ModLoader.addOverride("/gui/items.png", "/sdk/blockPlayerSpawner.png")).setItemName("itemPlayerSpawner");
        itemWeaponSpawnerGuns = new SdkItemUltimateTool(2529, blockWeaponSpawnerGuns).setIconIndex(ModLoader.addOverride("/gui/items.png", "/sdk/blockWeaponSpawnerGuns.png")).setItemName("itemWeaponSpawnerGuns");
        itemWeaponSpawnerNotGuns = new SdkItemUltimateTool(2533, blockWeaponSpawnerNotGuns).setIconIndex(ModLoader.addOverride("/gui/items.png", "/sdk/blockWeaponSpawnerNotGuns.png")).setItemName("itemWeaponSpawnerNotGuns");
    }
	
	public mod_SdkFps()
	{
		super();
		
		ModLoader.RegisterTileEntity(SdkTileEntityPlayerSpawner.class, "Player Spawner");
		ModLoader.RegisterTileEntity(SdkTileEntityWeaponSpawnerGuns.class, "Weapon Spawner Guns");
		ModLoader.RegisterTileEntity(SdkTileEntityWeaponSpawnerNotGuns.class, "Weapon Spawner Not Guns");
	}
	
	@Override
	public String Version()
	{
		return "1.4_01v3";
	}

	@Override
	public void SetRecipes() {
		SdkTools.setShapedRecipe(enableSpawnerRecipes, new ItemStack(itemPlayerSpawner, 1), new Object[] {
            " # ", "XXX", Character.valueOf('X'), Item.ingotIron, Character.valueOf('#'), Item.porkCooked
        });
		SdkTools.setShapedRecipe(enableSpawnerRecipes, new ItemStack(itemWeaponSpawnerGuns, 1), new Object[] {
            " # ", "XXX", Character.valueOf('X'), Item.ingotIron, Character.valueOf('#'), Item.gunpowder
        });
		SdkTools.setShapedRecipe(enableSpawnerRecipes, new ItemStack(itemWeaponSpawnerNotGuns, 1), new Object[] {
            " # ", "XXX", Character.valueOf('X'), Item.ingotIron, Character.valueOf('#'), Block.ladder
        });
	}
	
	@Override
    public void ModsLoaded()
    {
		super.ModsLoaded();
		buildWeaponMaps();
    }

	public static void buildWeaponMaps()
	{
		try
		{
			// Add guns to map
			weaponGunsIdItemMap.put((byte)0, (SdkItemGun)mod_SdkGuns.itemGunAk47);
	    	weaponGunsIdItemMap.put((byte)1, (SdkItemGun)mod_SdkGuns.itemGunMp5);
	    	weaponGunsIdItemMap.put((byte)2, (SdkItemGun)mod_SdkGuns.itemGunShotgun);
	    	weaponGunsIdItemMap.put((byte)3, (SdkItemGun)mod_SdkGuns.itemGunDeagle);
	    	weaponGunsIdItemMap.put((byte)4, (SdkItemGun)mod_SdkGuns.itemGunRocketLauncher);
	    	weaponGunsIdItemMap.put((byte)5, (SdkItemGun)mod_SdkGuns.itemGunRocketLauncherLaser);
	    	weaponGunsIdItemMap.put((byte)6, (SdkItemGun)mod_SdkGuns.itemGunSniper);
	    	weaponGunsIdItemMap.put((byte)7, (SdkItemGun)mod_SdkGuns.itemGunFlamethrower);
	    	weaponGunsIdItemMap.put((byte)8, (SdkItemGun)mod_SdkGuns.itemGunSg552);
	    	weaponGunsIdItemMap.put((byte)9, (SdkItemGun)mod_SdkGuns.itemGunMinigun);
	    	weaponGunsIdItemMap.put((byte)10, (SdkItemGun)mod_SdkGuns.itemGunLaser);
	    	weaponGunsIdItemMap.put((byte)11, (SdkItemGun)mod_SdkGuns.itemGunM4);
	    	
	    	// Add guns and ammo to allowed items
	    	for(byte i = 0; i < weaponGunsIdItemMap.size(); i++)
	    	{
	    		SdkItemGun weaponGun = weaponGunsIdItemMap.get(i);
	    		allowedItemIds.add(weaponGun.shiftedIndex);
				allowedItemIds.add(weaponGun.requiredBullet.shiftedIndex);
	    	}
	    	
	    	// Add gun textures
	    	for(int i = 0; i < weaponSpawnerGunsTextureIndices.length; i++)
	    		weaponGunsIdTextureMap.put(i, weaponSpawnerGunsTextureIndices[i]);
	    	
	    	// Add not guns to map
	    	weaponNotGunsIdItemMap.put((byte)0, mod_SdkGuns.itemSentry);
	    	weaponNotGunsIdItemMap.put((byte)1, mod_SdkGuns.itemGrenade);
	    	weaponNotGunsIdItemMap.put((byte)2, mod_SdkGuns.itemGrenadeSmoke);
	    	weaponNotGunsIdItemMap.put((byte)3, mod_SdkGuns.itemGrenadeStun);
	    	weaponNotGunsIdItemMap.put((byte)4, mod_SdkGuns.itemGrenadeSticky);
	    	weaponNotGunsIdItemMap.put((byte)5, mod_SdkGuns.itemGrenadeIncendiary);
	    	weaponNotGunsIdItemMap.put((byte)6, mod_SdkUtility.itemParachute);
	    	weaponNotGunsIdItemMap.put((byte)7, mod_SdkUtility.itemNightvisionGoggles);
	    	weaponNotGunsIdItemMap.put((byte)8, mod_SdkUtility.itemScubaTank);
	    	weaponNotGunsIdItemMap.put((byte)9, mod_SdkGrapplingHook.itemGrapplingHook);
	    	
	    	// Add not guns to allowed items
	    	for(byte i = 0; i < weaponNotGunsIdItemMap.size(); i++)
	    		allowedItemIds.add(weaponNotGunsIdItemMap.get(i).shiftedIndex);
	    	
	    	// Add not gun textures
	    	for(int i = 0; i < weaponSpawnerNotGunsTextureIndices.length; i++)
	    		weaponNotGunsIdTextureMap.put(i, weaponSpawnerNotGunsTextureIndices[i]);
	
	    	// Add spawners to allowed items
	    	allowedItemIds.add(itemPlayerSpawner.shiftedIndex);
	    	allowedItemIds.add(itemWeaponSpawnerGuns.shiftedIndex);
	    	allowedItemIds.add(itemWeaponSpawnerNotGuns.shiftedIndex);
	    	
	    	// Add spawners to admin items
	    	adminItemIds.add(itemPlayerSpawner.shiftedIndex);
	    	adminItemIds.add(itemWeaponSpawnerGuns.shiftedIndex);
	    	adminItemIds.add(itemWeaponSpawnerNotGuns.shiftedIndex);
		}
		catch(Throwable ex)
		{
			ModLoader.getLogger().throwing("mod_SdkFps", "buildWeaponMaps", ex);
			ModLoader.ThrowException("Error building weapon maps.", ex);
			return;
		}
	}
	
	@Override
	public void OnTickInGameTick(MinecraftServer minecraftServer)
	{
		
		// Restrict inventory
		if(restrictInventory && minecraftServer.worldMngr.playerEntities.size() > 0)
		{
			for(EntityPlayerMP e : new ArrayList<EntityPlayerMP>(minecraftServer.configManager.playerEntities))
			{
				InventoryPlayer inventory = e.inventory;
				
				Set<Integer> itemIds = new HashSet<Integer>();
				for(int i = 0; i < inventory.mainInventory.length; i++)
				{
					if(inventory.mainInventory[i] != null)
					{
						if(!allowedItemIds.contains(inventory.mainInventory[i].getItem().shiftedIndex))
						{
							inventory.mainInventory[i] = null;
						}
						else if(itemIds.contains(inventory.mainInventory[i].itemID))
						{
							if(inventory.mainInventory[i].getItem() instanceof SdkItemGun)
							{
								SdkItemGun itemGun = (SdkItemGun)inventory.mainInventory[i].getItem();
								inventory.mainInventory[i] = new ItemStack(itemGun.requiredBullet, itemGun.requiredBullet.maxStackSize);
							}
						}
						else
						{
							itemIds.add(inventory.mainInventory[i].itemID);
						}
					}
				}
			}
		}
		// Set mineable
		if(blocksMineable != areBlocksMineable)
			setBlocksMineable(blocksMineable);

		// Set explodable
		if(blocksExplodable != areBlocksExplodable)
			setBlocksExplodable(blocksExplodable);

		// Set burnable
		if(blocksBurnable != areBlocksBurnable)
			setBlocksBurnable(blocksBurnable);
	}
	
	@Override
	public void HandleLogin(EntityPlayerMP entityPlayerMp)
	{
		List loadedMods = ModLoader.getLoadedMods();
		for(int i = 0; i < loadedMods.size(); i++)
			if(loadedMods.get(i) instanceof mod_SdkGuns)
				((mod_SdkGuns)loadedMods.get(i)).sendProperties(entityPlayerMp);
		
		sendProperties(entityPlayerMp);
		setSpawnPoint(entityPlayerMp, false);
	}

	public void sendProperties(EntityPlayerMP entityPlayerMp)
	{
		int[] dataInt = new int[11];
		dataInt[0] = blocksMineable ? 1 : 0;
		dataInt[1] = blocksExplodable ? 1 : 0;
		dataInt[2] = blocksBurnable ? 1 : 0;
		dataInt[3] = enablePlayerSpawners ? 1 : 0;
        dataInt[4] = useRandomPlayerSpawner ? 1: 0;
		dataInt[5] = enableWeaponSpawners ? 1 : 0;
		dataInt[6] = enableSpawnerRecipes ? 1 : 0;
		dataInt[7] = restrictInventory ? 1 : 0;
		dataInt[8] = respawnWithPistol ? 1 : 0;
		dataInt[9] = weaponSpawnerRespawnDelay;
		dataInt[10] = playersDropInventory ? 1 : 0;
		
		Packet200ModLoader packet200ModLoader = new Packet200ModLoader();
		packet200ModLoader.packetType = 0;
		packet200ModLoader.dataInt = dataInt;
		ModLoaderMp.SendPacketTo(this, entityPlayerMp, packet200ModLoader);
	}

	private void setBlocksMineable(boolean setMineable)
	{
		try
		{
			if(!setMineable)
			{
				// Make unmineable
				blockHardness = new float[Block.blocksList.length];
				
				for(int i = 0; i < Block.blocksList.length; i++)
				{
					if(Block.blocksList[i] != null)
					{
						blockHardness[i] = Block.blocksList[i].blockHardness;
						
						if(i != Block.glass.blockID || !mod_SdkGuns.bulletsDestroyGlass)
							Block.blocksList[i].blockHardness = 6000000F;
					}
				}
			}
			else if (blockHardness != null)
			{
				// Make mineable
				for(int i = 0; i < Block.blocksList.length; i++)
				{
					if(Block.blocksList[i] != null)
					{
						Block.blocksList[i].blockHardness = blockHardness[i];
					}
				}
			}
			
			areBlocksMineable = setMineable;
		}
        catch(Exception ex)
        {
        	ModLoader.getLogger().throwing("mod_SdkFps", "setBlocksMineable", ex);
			SdkTools.ThrowException(String.format("Error setting blocks mineable: {0}.", setMineable), ex);
			return;
        }
	}
	
	private void setBlocksExplodable(boolean setExplodable)
	{
		try
		{
			if(!setExplodable)
			{
				// Make unexplodable
				blockResistance = new float[Block.blocksList.length];
				
				for(int i = 0; i < Block.blocksList.length; i++)
				{
					if(Block.blocksList[i] != null)
					{
						blockResistance[i] = Block.blocksList[i].blockResistance;
						
						if(i != Block.glass.blockID || !mod_SdkGuns.bulletsDestroyGlass)
							Block.blocksList[i].blockResistance = 6000000F;
					}
				}
			}
			else if (blockResistance != null)
			{
				// Make explodable
				for(int i = 0; i < Block.blocksList.length; i++)
				{
					if(Block.blocksList[i] != null)
					{
						Block.blocksList[i].blockResistance = blockResistance[i];
					}
				}
			}
			
			areBlocksExplodable = setExplodable;
		}
        catch(Exception ex)
        {
        	ModLoader.getLogger().throwing("mod_SdkFps", "setBlocksExplodable", ex);
			SdkTools.ThrowException(String.format("Error setting blocks explodable: {0}.", setExplodable), ex);
			return;
        }
	}

	private void setBlocksBurnable(boolean setBurnable)
	{
		try
		{
			if(!setBurnable)
			{
				// Make unburnable
				blockChanceToEncourageFire = ((int[])ModLoader.getPrivateValue(BlockFire.class, Block.fire, 0)).clone();
				blockAbilityToCatchFire = ((int[])ModLoader.getPrivateValue(BlockFire.class, Block.fire, 1)).clone();
				
				ModLoader.setPrivateValue(BlockFire.class, Block.fire, 0, new int[blockChanceToEncourageFire.length]);
				ModLoader.setPrivateValue(BlockFire.class, Block.fire, 1, new int[blockAbilityToCatchFire.length]);
				
				// Leave some things burnable
		        SdkTools.setBurnRate(Block.tnt.blockID, 15, 100);
				mod_SdkGuns.setBurnRates();
			}
			else if (blockChanceToEncourageFire != null && blockAbilityToCatchFire != null)
			{
				// Make burnable
				ModLoader.setPrivateValue(BlockFire.class, Block.fire, 0, blockChanceToEncourageFire);
				ModLoader.setPrivateValue(BlockFire.class, Block.fire, 1, blockAbilityToCatchFire);
			}
			
			areBlocksBurnable = setBurnable;
		}
        catch(Exception ex)
        {
        	ModLoader.getLogger().throwing("mod_SdkFps", "setBlocksBurnable", ex);
			SdkTools.ThrowException(String.format("Error setting blocks burnable: {0}.", setBurnable), ex);
			return;
        }
	}

	public static void registerPlayerSpawner(int i, int j, int k)
	{
		synchronized(mod_SdkFps.class) {
			SdkPoint3d<Integer> spawnPoint = new SdkPoint3d<Integer>(i, j, k);
			
			if(!spawnPoints.contains(spawnPoint)) {
				spawnPoints.add(spawnPoint);
			}
		}
	}

	public static void unregisterPlayerSpawner(int i, int j, int k)
	{
		synchronized(mod_SdkFps.class) {
			SdkPoint3d<Integer> spawnPoint = new SdkPoint3d<Integer>(i, j, k);
			
			if(spawnPoints.contains(spawnPoint)) {
				spawnPoints.remove(new SdkPoint3d<Integer>(i, j, k));
			}
		}
	}
	
	public static void setSpawnPoint(EntityPlayer entityPlayer, boolean respawn)
	{
		setSpawnPoint(entityPlayer, entityPlayer.posX, entityPlayer.posY, entityPlayer.posZ, respawn);
	}
	
	public static void setSpawnPoint(EntityPlayer entityPlayer, double prevX, double prevY, double prevZ, boolean respawn)
	{
		synchronized(mod_SdkFps.class)
		{
			if(respawn || mod_SdkGuns.playerInventoryEmpty(entityPlayer.inventory))
			{
				if(enablePlayerSpawners && spawnPoints.size() > 0)
				{
					SdkPoint3d<Integer> spawnPoint;
					if(useRandomPlayerSpawner)
						spawnPoint = spawnPoints.get(SdkTools.random.nextInt(spawnPoints.size()));
					else
					{
						double minDistanceSquared = Double.MAX_VALUE;
						int minIndex = 0;
						for(int i = 0; i < spawnPoints.size(); i++)
						{
							SdkPoint3d<Integer> tempSpawnPoint = spawnPoints.get(i);
							
							double distanceX = tempSpawnPoint.x - prevX;
							double distanceY = tempSpawnPoint.y - prevY;
							double distanceZ = tempSpawnPoint.z - prevZ;
							
							double distanceSquared = distanceX * distanceX + distanceY * distanceY + distanceZ * distanceZ;
							if(distanceSquared < minDistanceSquared)
							{
								minDistanceSquared = distanceSquared;
								minIndex = i;
							}
						}
						
						spawnPoint = spawnPoints.get(minIndex);
					}
					
					entityPlayer.setPosition(spawnPoint.x + 0.5D, spawnPoint.y + 1.6825F, spawnPoint.z + 0.5D);
				}
				
				if(respawnWithPistol)
				{
					setPlayerInventory(entityPlayer);
				}
			}
		}
	}

	private static void setPlayerInventory(EntityPlayer entityPlayer)
	{
		setPlayerInventoryItem(entityPlayer, 0, new ItemStack(mod_SdkGuns.itemGunDeagle));
		setPlayerInventoryItem(entityPlayer, 9, new ItemStack(mod_SdkGuns.itemBulletMedium, mod_SdkGuns.itemBulletMedium.maxStackSize));
		setPlayerInventoryItem(entityPlayer, 10, new ItemStack(mod_SdkGuns.itemBulletMedium, mod_SdkGuns.itemBulletMedium.maxStackSize));
	}
	
	private static boolean setPlayerInventoryItem(EntityPlayer entityPlayer, int index, ItemStack itemStack)
	{
		if(entityPlayer.inventory.mainInventory[index] == null)
		{
			entityPlayer.inventory.mainInventory[index] = itemStack;
			return true;
		}
		else
		{
			return entityPlayer.inventory.addItemStackToInventory(itemStack);
		}
	}

	@Override
	public void OnMainMenu() { }
}
